import os
import shutil
import time

from selenium import webdriver
from selenium.webdriver import Keys
from webdriver_manager.chrome import ChromeDriverManager
from selenium.webdriver.support.ui import WebDriverWait

whatsapp_url: str = "https://web.whatsapp.com/"

sleep_time = 3

input_search_xpath = "//*[@id=\"side\"]/div[1]/div/label/div/div[2]"
input_message_xpath = "//*[@id=\"main\"]/footer/div[1]/div/span[2]/div/div[2]/div[1]/div/div[2]"
open_group_xpath = "//*[@id=\"main\"]/header/div[2]/div[1]/div/span"
qr_xpath = "//*[@id=\"app\"]/div[1]/div/div[2]/div[1]/div/div[2]/div/canvas"


def select_contact_xpath(contact) -> str:
    return '//span[@title="{}"]'.format(contact)


def assert_correct_contact(group: str, group_title_element):
    assert group_title_element.text == group


def send_message(group: str, message: str, headed: bool):
    try:
        options = webdriver.ChromeOptions()
        options.add_argument('--user-data-dir=./User_Data')

        if not headed:
            options.add_argument('--headless')

        driver = webdriver.Chrome(ChromeDriverManager().install(), chrome_options=options)

        driver.get(whatsapp_url)

        input_box_search = WebDriverWait(driver, 50) \
            .until(lambda driver: driver.find_element_by_xpath(input_search_xpath))
        input_box_search.click()
        input_box_search.send_keys(group)

        selected_contact = WebDriverWait(driver, 50) \
            .until(lambda driver: driver.find_element_by_xpath(select_contact_xpath(group)))
        selected_contact.click()

        group_title_element = WebDriverWait(driver, 50) \
            .until(lambda driver: driver.find_element_by_xpath(open_group_xpath))
        assert_correct_contact(group, group_title_element)

        message_box = WebDriverWait(driver, 50) \
            .until(lambda driver: driver.find_element_by_xpath(input_message_xpath))
        message_box.send_keys(message + Keys.ENTER)

        # sleep is needed to wait for sending the message
        time.sleep(sleep_time)

    finally:
        driver.quit()


def login():
    dirname = os.path.dirname(__file__)

    relative_path_to_user_data_folder = "../../User_Data"
    user_data_folder_path = os.path.join(dirname, relative_path_to_user_data_folder)

    if os.path.isdir(user_data_folder_path):
        shutil.rmtree(user_data_folder_path)

    try:
        options = webdriver.ChromeOptions()
        options.add_argument('--user-data-dir=./User_Data')

        driver = webdriver.Chrome(ChromeDriverManager().install(), chrome_options=options)

        driver.get(whatsapp_url)

        WebDriverWait(driver, 50) \
            .until(lambda driver: driver.find_element_by_xpath(qr_xpath))

        WebDriverWait(driver, 50) \
            .until_not(lambda driver: driver.find_element_by_xpath(qr_xpath))

        time.sleep(sleep_time)

    finally:
        driver.quit()



import requests
import whatajoke.joke.url_helper as url_helper
import whatajoke.joke.joke_formatter as joke_formatter


def get_joke(category: str, flag: str) -> str:
    """

    :rtype: object
    """
    # Get formatted url
    url = url_helper.get_url(category, flag)

    # Send request
    response = requests.get(url)
    # print(response.content)

    if response.status_code == 200:
        # Format joke
        joke = joke_formatter.format_joke(response.content)
    else:
        raise Exception("Ups! Something went wrong on joke request.")

    print(joke)
    return joke



import configparser

configuration_file = 'config.txt'

# Read configuration file
config = configparser.ConfigParser()
config.read(configuration_file)


def get_url(category: str, flag: str) -> str:
    base_url = config['config']['api_url']

    formatted_category = get_category(category)
    query = get_query(flag)

    full_url = base_url + formatted_category + query
    return full_url


def get_category(category: str) -> str:
    categories = config['config']['categories']

    capitalized_category = str(category).capitalize()

    if capitalized_category in categories:
        return capitalized_category
    else:
        raise Exception('Invalid category')


def get_query(flag: str) -> str:
    base_query = "?blacklistFlags="

    flags = config['config']['flags']

    if flag in flags:
        # When flag is 'none', api doesn't need query params
        if flag == 'none':
            return ''
        full_query = base_query + flag
    else:
        raise Exception('Invalid flag')

    return full_query



import click
import whatajoke.service


@click.command()
@click.option("--group", "-g",
              required=True,
              prompt="Name of the group or person to send the joke to",
              help="Name of the group or person to send the joke to!")
@click.option("--category", "-c",
              type=click.Choice(["any", "programming", "misc", "dark", "pun", "spooky", "christmas"],
                                case_sensitive=False),
              default="any",
              help="Category of the joke. Default value is \"any\".")
@click.option("--flag", "-f",
              type=click.Choice(["none", "nsfw", "religious", "political", "racist", "sexist", "explicit"],
                                case_sensitive=False),
              default="none",
              help="Flag to add to category. Default value is \"none\".")
@click.option("--headed", "-h",
              is_flag=True,
              help="Runs in headed mode.")
def send(group, category, flag, headed):
    try:
        # Parse choice to String
        category = str(category)
        flag = str(flag)
        group = str(group)

        whatajoke.service.send_joke(group, category, flag, headed)

    except Exception as e:
      print(e)

    finally:
        print("See you soon!")


import json


def is_two_part(content: str) -> bool:
    return content['type'] == "twopart"


def format_one_part_joke(raw_joke: json) -> str:
    return raw_joke["joke"]


def format_two_part_joke(raw_joke: json) -> str:
    setup = raw_joke["setup"]
    delivery = raw_joke["delivery"]
    fill = " "

    full_joke = f'''{setup}{fill}{delivery}'''

    return full_joke


def format_joke(http_response_content: json) -> str:
    parsed_content = json.loads(http_response_content.decode('utf-8'))

    joke: str

    if is_two_part(parsed_content):
        joke = format_two_part_joke(parsed_content)
    else:
        joke = format_one_part_joke(parsed_content)

    return joke
